''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Microsoft Corporation. All rights reserved.
' THIS CODE IS MADE AVAILABLE AS IS, WITHOUT WARRANTY OF ANY KIND. THE ENTIRE 
' RISK OF THE USE OR THE RESULTS FROM THE USE OF THIS CODE REMAINS WITH THE 
' USER. USE AND REDISTRIBUTION OF THIS CODE, WITH OR WITHOUT MODIFICATION, IS 
' HEREBY PERMITTED.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' This script converts an MSDE log into a text file or displays the content of 
' the MSDE log on the console screen. This script can be run only on the local
' computer where the MSDE database files are stored.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

' Declare constants needed.
Const adBoolean     = 11
Const adDBTimeStamp = 135
Const adVarWChar    = 200
Const adVarChar     = 202


MSDEToText

Sub MSDEToText()

    ' Declare objects needed.
    Dim args              ' A WScript Arguments object
    Dim dbName            ' A String
    Dim tableName         ' A String
    Dim outFileName       ' A String
    Dim outFile           ' A TextStream object
    Dim fso               ' A FileSystem object
    Dim cn                ' An ADODB Connection object
    Dim ProductVer        ' version string

    Set args = WScript.Arguments
    If args.Count <> 3 And args.Count <> 4 Then
        Usage()
    Else

        ProductVer = args.Item(0)
        If ProductVer <> "2006" And ProductVer <> "TMG" Then
            Usage()
        End If
        DbName = args.Item(1)
        tableName  = args.Item(2)
   
        Set fso = CreateObject("Scripting.FileSystemObject")
        If args.Count = 4 Then
           outFileName = args.Item(3)
           Set outFile = fso.CreateTextFile(outFileName, True)
        Else
            outFileName = ""
            Set outFile = WScript.StdOut
        End If
        Set fso = Nothing
    End If


    Set cn = CreateObject("ADODB.Connection")

    cn.ConnectionTimeout = 25
    cn.Provider = "SQLOLEDB"
    cn.Properties("Data Source").Value = "(local)\msfw"
    cn.Properties("Initial Catalog").Value = DbName
    cn.Properties("Integrated Security").Value = "SSPI"

    cn.Open

    Call PrintTable(ProductVer, cn, tableName, outFile)

    Set outFile = Nothing
    WScript.Echo "Done!"
End Sub

Sub PrintTable(ProductVer, cn, tableName, outFile)
    Dim recSet           ' An ADODB Recordset
    Dim field            ' An ADODB Field
    Dim queryText        ' A String
    Dim rowString        ' A String
    Dim rowValue         ' A String   
    Dim boolValue        ' A Boolean
    Dim firstTime        ' A Boolean
    Dim logCollection    ' A Scripting Dictionary object
    Dim logItem          ' An item in a Scripting Dictionary object
    Dim columnsArray     ' An array
    Dim columnString     ' A String
    Dim i                ' Integers
    Dim fieldName        ' A String
    Dim separator        ' A String    

    columnString = ""
    firstTime = True

    Set logCollection = CreateObject("Scripting.Dictionary")
  
    Select Case Trim(UCase(tableName))
        Case "WEBPROXYLOG"
            Select Case ProductVer 
                Case "2006"
                    BuildWebProxyLogCollection2006(logCollection)
                Case "TMG"
                    BuildWebProxyLogCollection2008(logCollection)
            End Select
        Case "FIREWALLLOG"
            Select Case ProductVer 
                Case "2006"
                    BuildFirewallLogCollection2006(logCollection)
                Case "TMG"
                    BuildFirewallLogCollection2008(logCollection)
            End Select          
    End Select
    columnsArray = logCollection.Keys
    queryText = "SELECT * FROM " & tableName

    Set recSet = CreateObject("ADODB.Recordset")

    ' Open the recordset.
    recSet.Open queryText, cn

    ' Move to the first row.
    recSet.MoveFirst
   
    rowString = "#"
    For i = 0 To logCollection.Count - 1
        columnString = columnString & CStr(columnsArray(i)) & vbTab
    Next
  
    Do While Not recSet.EOF
        With recSet
            rowValue = ""
            rowString = ""

            ' Concatenate the column values for each record.
            For i = 0 to logCollection.Count - 1
                logItem = logCollection.Item(columnsArray(i))
                'WScript.Echo(logItem)

                ' Retrieve the appropriate name of MSDE column as a function
                ' of the real log column.
                Set field = recSet.Fields.Item(logItem) 
                If Not IsNull(field.Value) Then
                    fieldName = Trim(UCase(field.Name))
                    Select Case field.Type
                        ' Catch ADO Boolean type values, convert to VB string.
                        Case adBoolean:
                            boolValue = field.Value
                            rowValue = CStr(boolValue)
                        Case adDBTimeStamp:            
                            If fieldName = "GMTLOGTIME" Then
                                separator = " "
                            Else
                                separator = vbTab
                            End If
                            rowValue = ConvertDateToText(field.Value) & separator & ConvertTimeToText(field.Value)
                        Case adVarChar:
                            rowValue = ValueIfLegal(field.value)
                        Case adVarWChar:
                            rowValue = ValueIfLegal(field.value)
                        Case Else:
                            rowValue = field.Value
                            Select Case fieldName
                                Case "RESULTCODE", "ERRORINFO", "CACHEINFO":
                                    rowValue = "0x" & Hex(rowValue)
                                Case "CLIENTIP", "DESTHOSTIP", "ORIGINALCLIENTIP", "SOURCEIP", "DESTINATIONIP":
                                    rowValue = ConvertIP(rowValue)
                                    if fieldName = "SOURCEIP" Then
                                        Set field = recSet.Fields.Item("SourcePort")                                        
                                        rowValue = rowValue & ":" & field.Value
                                    ElseIf fieldName = "DESTINATIONIP" Then
                                        Set field = recSet.Fields.Item("DestinationPort")                                        
                                        rowValue = rowValue & ":" & field.Value
                                    End If
                                Case Else:
                            End Select
                    End Select        
                Else
                    ' Catch NULL values, convert to string "NULL" for display.
                    rowValue = "-"
                End If   
				If i = logCollection.Count - 1 Then
				    rowString = rowString & rowValue
				Else
                    rowString = rowString & rowValue & vbTab
			    End If
            Next ' End of inner loop
            If firstTime Then
                Select Case ProductVer 
                    Case "2006"
                        outFile.writeLine("#Software: Microsoft (R) ISA Server 2006")
                    Case "TMG"
                        outFile.writeLine("#Software: Microsoft (R) Forefront TMG")
                End Select
                outFile.writeLine("#Version: 2.0")
                outFile.writeLine("#Date: " & ConvertDateToText(recSet.Fields.Item("logTime")) & " 00:00:00")
                outFile.WriteLine("#Fields: " & columnString)
                WScript.Echo("")
                firstTime = False
            End If
            outFile.WriteLine(rowString)
            rowString = ""
        End With
        recSet.MoveNext
    Loop                    ' End of outer loop
End Sub

Private Function ConvertIP(rowValue)
    ' test for GUID (in TMG, IPs are stored in a format similar to GUID)
    If Len(CStr(rowValue)) = 38 Then
        ConvertIP = IPFromGUID(rowValue)
    Else
        ConvertIP = IPFromDbl(rowValue)
    End If
End Function

Private Function IPFromDbl(rowValue)
    Dim ipDouble         ' Double
    Dim dot              ' string
    Dim count,octet      ' integers

    ipDouble = CDbl(rowValue)
    IPFromDbl= ""
    dot = "" 
    For count = 1 To 4
        ipDouble = (Fix(ipDouble)) / 256
        octet = 256 * (ipDouble - Fix(ipDouble))
        IPFromDbl = CStr(octet) & dot & IPFromDbl
        dot = "."
    Next

End Function

Private Function IPFromGUID(rowValue)
    Dim strIp ' a string
    Dim count ' integer
    Dim arr   ' array
    
    strIp = CStr(rowValue)

    If UCase(Mid(strIp, 11, 4)) = "FFFF" Then
        ' IPv4
        IPFromGUID = IPFromDbl("&H" & UCase(Mid(strIp, 2, 8)))
        Exit Function
    End If
    
    ' IPv6
    IPFromGUID = ""
    ' array of indexes (for IPv6 parts) in the GUID string 
    arr = Array(2,6,11,16,21,26,30,34)
    For count = LBound(arr) to UBound(arr)
        IPFromGUID = IPFromGUID & Mid(strIp, arr(count), 4) & ":"
    Next
    ' remove trailing ":"
    IPFromGUID = Left(IPFromGUID, Len(IPFromGUID)-1)
End Function
                                          
Sub BuildWebProxyLogCollection2006(wpLogCollection)
    
    wpLogCollection.Add "c-ip", "ClientIP"
    wpLogCollection.Add "cs-username", "ClientUserName"
    wpLogCollection.Add "c-agent","ClientAgent"
    wpLogCollection.Add "sc-authenticated", "ClientAuthenticate"
    wpLogCollection.Add "date" & vbTab & "time", "logTime"
    wpLogCollection.Add "s-svcname", "service"
    wpLogCollection.Add "s-computername", "servername"
    wpLogCollection.Add "cs-referred", "referredserver"
    wpLogCollection.Add "r-host", "DestHost"
    wpLogCollection.Add "r-ip", "DestHostIP"
    wpLogCollection.Add "r-port", "DestHostPort"
    wpLogCollection.Add "time-taken", "processingtime"
    wpLogCollection.Add "sc-bytes", "bytesrecvd"
    wpLogCollection.Add "cs-bytes", "bytessent"
    wpLogCollection.Add "cs-protocol", "protocol"
    wpLogCollection.Add "cs-transport", "transport"
    wpLogCollection.Add "s-operation", "operation"
    wpLogCollection.Add "cs-uri", "uri"
    wpLogCollection.Add "cs-mime-type", "mimetype"
    wpLogCollection.Add "s-object-source", "objectsource"
    wpLogCollection.Add "sc-status", "resultcode"
    wpLogCollection.Add "s-cache-info", "CacheInfo"
    wpLogCollection.Add "rule","rule"
    wpLogCollection.Add "FilterInfo","FilterInfo"
    wpLogCollection.Add "cs-Network", "SrcNetwork"
    wpLogCollection.Add "sc-Network", "DstNetwork"
    wpLogCollection.Add "error-info", "ErrorInfo"
    wpLogCollection.Add "action", "Action"
    wpLogCollection.Add "GmtLogTime", "GmtLogTime"
    wpLogCollection.Add "AuthenticationServer", "AuthenticationServer"
End Sub

Sub BuildWebProxyLogCollection2008(wpLogCollection)
    
    wpLogCollection.Add "c-ip", "ClientIP"
    wpLogCollection.Add "cs-username", "ClientUserName"
    wpLogCollection.Add "c-agent","ClientAgent"
    wpLogCollection.Add "sc-authenticated", "ClientAuthenticate"
    wpLogCollection.Add "date" & vbTab & "time", "logTime"
    wpLogCollection.Add "s-svcname", "service"
    wpLogCollection.Add "s-computername", "servername"
    wpLogCollection.Add "cs-referred", "referredserver"
    wpLogCollection.Add "r-host", "DestHost"
    wpLogCollection.Add "r-ip", "DestHostIP"
    wpLogCollection.Add "r-port", "DestHostPort"
    wpLogCollection.Add "time-taken", "processingtime"
    wpLogCollection.Add "sc-bytes", "bytesrecvd"
    wpLogCollection.Add "cs-bytes", "bytessent"
    wpLogCollection.Add "cs-protocol", "protocol"
    wpLogCollection.Add "cs-transport", "transport"
    wpLogCollection.Add "s-operation", "operation"
    wpLogCollection.Add "cs-uri", "uri"
    wpLogCollection.Add "cs-mime-type", "mimetype"
    wpLogCollection.Add "s-object-source", "objectsource"
    wpLogCollection.Add "sc-status", "resultcode"
    wpLogCollection.Add "s-cache-info", "CacheInfo"
    wpLogCollection.Add "rule","rule"
    wpLogCollection.Add "FilterInfo","FilterInfo"
    wpLogCollection.Add "cs-Network", "SrcNetwork"
    wpLogCollection.Add "sc-Network", "DstNetwork"
    wpLogCollection.Add "error-info", "ErrorInfo"
    wpLogCollection.Add "action", "Action"
    wpLogCollection.Add "GmtLogTime", "GmtLogTime"
    wpLogCollection.Add "AuthenticationServer", "AuthenticationServer"
    wpLogCollection.Add "IPS scan result", "ipsScanResult"
    wpLogCollection.Add "IPS signature", "ipsSignature"
    wpLogCollection.Add "ThreatName", "ThreatName"
    wpLogCollection.Add "MalwareInspectionAction", "MalwareInspectionAction"
    wpLogCollection.Add "MalwareInspectionResult", "MalwareInspectionResult"
    wpLogCollection.Add "UrlCategory", "UrlCategory"
    wpLogCollection.Add "MalwareInspectionContentDeliveryMethod", "MalwareInspectionContentDeliveryMethod"
    wpLogCollection.Add "mi-uagarrayid", "UagArrayId"
    wpLogCollection.Add "sc-uagversion", "UagVersion"
    wpLogCollection.Add "mi-uagmoduleid", "UagModuleId"
    wpLogCollection.Add "sc-uagid", "UagId"
    wpLogCollection.Add "mi-uagseverity", "UagSeverity"
    wpLogCollection.Add "mi-uagtype", "UagType"
    wpLogCollection.Add "sc-uageventname", "UagEventName"
    wpLogCollection.Add "mi-uagsessionid", "UagSessionId"
    wpLogCollection.Add "mi-uagtrunkname", "UagTrunkName"
    wpLogCollection.Add "mi-uagservicename", "UagServiceName"
    wpLogCollection.Add "sc-uagerrorcode", "UagErrorCode"
    wpLogCollection.Add "MalwareInspectionDuration", "MalwareInspectionDuration"
    wpLogCollection.Add "MalwareInspectionThreatLevel", "MalwareInspectionThreatLevel"
    wpLogCollection.Add "internal-service-info", "InternalServiceInfo"
    wpLogCollection.Add "NIS application protocol", "ipsApplicationProtocol"
    wpLogCollection.Add "NAT Address", "NATAddress"
    wpLogCollection.Add "UrlCategorizationReason", "UrlCategorizationReason"
    wpLogCollection.Add "Session Type", "SessionType"
    wpLogCollection.Add "URL Destination Host", "UrlDestHost"
    wpLogCollection.Add "Source Port", "SrcPort"
End Sub


Sub BuildFirewallLogCollection2006(fwLogCollection)
    
    fwLogCollection.Add "computer", "servername"
    fwLogCollection.Add "date" & vbTab & "time", "logTime"
    fwLogCollection.Add "IP protocol","protocol"
    fwLogCollection.Add "source", "SourceIP"                
    fwLogCollection.Add "destination","DestinationIP"
    fwLogCollection.Add "original client IP", "OriginalClientIP"
    fwLogCollection.Add "source network","SourceNetwork"
    fwLogCollection.Add "destination network", "DestinationNetwork"
    fwLogCollection.Add "action","Action"
    fwLogCollection.Add "status", "resultcode"
    fwLogCollection.Add "rule","rule"
    fwLogCollection.Add "application protocol", "ApplicationProtocol"
    fwLogCollection.Add "bidirectional", "Bidirectional"
    fwLogCollection.Add "bytes sent","bytessent"
    fwLogCollection.Add "bytes sent intermediate", "bytessentDelta"
    fwLogCollection.Add "bytes received", "bytesrecvd"        
    fwLogCollection.Add "bytes received intermediate","bytesrecvdDelta"
    fwLogCollection.Add "connection time", "connectiontime"
    fwLogCollection.Add "connection time intermediate", "connectiontimeDelta"
    fwLogCollection.Add "source proxy", "SourceProxy"
    fwLogCollection.Add "destination proxy", "DestinationProxy"
    fwLogCollection.Add "source name", "SourceName"
    fwLogCollection.Add "destination name", "DestinationName"
    fwLogCollection.Add "username", "ClientUserName"
    fwLogCollection.Add "agent","ClientAgent"
    fwLogCollection.Add "session ID","sessionid"
    fwLogCollection.Add "connection ID","connectionid"
    fwLogCollection.Add "interface", "Interface"
    fwLogCollection.Add "IP Header", "IPHeader"
    fwLogCollection.Add "protocol payload", "Payload"
    fwLogCollection.Add "GmtLogTime", "GmtLogTime"
End Sub

Sub BuildFirewallLogCollection2008(fwLogCollection)
    
    fwLogCollection.Add "computer", "servername"
    fwLogCollection.Add "date" & vbTab & "time", "logTime"
    fwLogCollection.Add "IP protocol","protocol"
    fwLogCollection.Add "source", "SourceIP"    
    fwLogCollection.Add "sourceport", "SourcePort"                            
    fwLogCollection.Add "destination","DestinationIP"
    fwLogCollection.Add "destinationport","DestinationPort"
    fwLogCollection.Add "original client IP", "OriginalClientIP"
    fwLogCollection.Add "source network","SourceNetwork"
    fwLogCollection.Add "destination network", "DestinationNetwork"
    fwLogCollection.Add "action","Action"
    fwLogCollection.Add "status", "resultcode"
    fwLogCollection.Add "rule","rule"
    fwLogCollection.Add "application protocol", "ApplicationProtocol"
    fwLogCollection.Add "bidirectional", "Bidirectional"
    fwLogCollection.Add "bytes sent","bytessent"
    fwLogCollection.Add "bytes sent intermediate", "bytessentDelta"
    fwLogCollection.Add "bytes received", "bytesrecvd"        
    fwLogCollection.Add "bytes received intermediate","bytesrecvdDelta"
    fwLogCollection.Add "connection time", "connectiontime"
    fwLogCollection.Add "connection time intermediate", "connectiontimeDelta"
    fwLogCollection.Add "destination name", "DestinationName"
    fwLogCollection.Add "username", "ClientUserName"
    fwLogCollection.Add "agent","ClientAgent"
    fwLogCollection.Add "session ID","sessionid"
    fwLogCollection.Add "connection ID","connectionid"
    fwLogCollection.Add "interface", "Interface"
    fwLogCollection.Add "IP Header", "IPHeader"
    fwLogCollection.Add "protocol payload", "Payload"
    fwLogCollection.Add "GmtLogTime", "GmtLogTime"
    fwLogCollection.Add "IPS scan result", "ipsScanResult"
    fwLogCollection.Add "IPS signature", "ipsSignature"
    fwLogCollection.Add "NAT Address", "NATAddress"
    fwLogCollection.Add "fwc-client-fqdn", "FwcClientFqdn"
    fwLogCollection.Add "fwc-app-path", "FwcAppPath"
    fwLogCollection.Add "fwc-app-sha1-hash", "FwcAppSHA1Hash"
    fwLogCollection.Add "fwc-app-trust-state", "FwcAppTrusState"
    fwLogCollection.Add "fwc-app-internal-name", "FwcAppInternalName"
    fwLogCollection.Add "fwc-app-product-name", "FwcAppProductName"
    fwLogCollection.Add "fwc-app-product-version", "FwcAppProductVersion"
    fwLogCollection.Add "fwc-app-file-vrsion", "FwcAppFileVersion"
    fwLogCollection.Add "fwc-app-original-file-name", "FwcAppOrgFileName"
    fwLogCollection.Add "internal-service-info", "InternalServiceInfo"
    fwLogCollection.Add "NIS application protocol", "ipsApplicationProtocol"
    fwLogCollection.Add "fwc-version", "FwcVersion"
End Sub


Private Function ConvertDateToText(msdeTime)
    Dim monthStr
    Dim yearStr
    Dim dayStr
    Dim dateVal

    dateVal = CDate(msdeTime)
    
    monthStr = AddZero(Month(dateVal))    
    dayStr = AddZero(Day(dateVal))
    yearStr = cStr(Year(dateVal))
    
    ConvertDateToText = yearStr & "-" & monthStr  & "-" & dayStr 
End Function

Private Function AddZero(theObj)
    If(Len(CStr(theObj)) < 2) Then
        AddZero = "0" & CStr(theObj)
    Else
        AddZero = CStr(theObj)
    End If
End Function

Private Function convertTimeToText(msdeTime)
    Dim hourStr
    Dim minStr
    Dim secStr
    
    Dim dateVal
    
    dateVal = CDate(msdeTime)
    hourStr = AddZero(Hour(dateVal))
    minStr = AddZero(Minute(dateVal))
    secStr = AddZero(Second(dateVal))
    convertTimeToText = hourStr & ":" & minStr & ":" & secStr
End Function

Private Sub PrintCollection(coll)
    For Each item In coll
        Wscript.Echo item
    Next
End Sub

Private Function ContainsIllegalChar(str)
    ContainsIllegalChar = false
    If InStr(str, vbCR) <> 0 Or _
       InStr(str, vbLF) <> 0 Or _
       InStr(str, vbTab) <> 0 Or _
       InStr(str, vbNullChar) <> 0 Then
        ContainsIllegalChar = true
    End If
End Function

Private Function ValueIfLegal(str)
    ValueIfLegal = str
    If ContainsIllegalChar(str) Then
        ValueIfLegal = "Data is not valid"
    End If
End Function


Sub Usage()
    WScript.Echo "Usage:" & VbCrLf _
        & "  CScript " & WScript.ScriptName & " ProductVersion DataBase Table [OutputFile]" & VbCrLf _
        & "" & VbCrLf _
        & "  ProductVersion  - ISA or TMG version. Possible values:" & VbCrLf _
        & "                        2006 (ISA 2006)" & VbCrLf _
        & "                        TMG  (Forefront TMG)" & VbCrLf _
        & "  DataBase        - The file name of the MSDE database without the mdf extension" & VbCrLf _
        & "  Table           - The type of log. Possible values:" & VbCrLf _
        & "                        WebProxyLog" & VbCrLf _
        & "                        FirewallLog" & VbCrLf _
        & "  OutputFile      - The name of the output file" 

    WScript.Quit(0)
End Sub
